//go:build e2e
// +build e2e

/*
Copyright 2019 The Knative Authors
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package conformance

import (
	"context"
	"flag"
	"log"
	"os"
	"strings"
	"testing"

	"knative.dev/pkg/system"

	"knative.dev/eventing/test"
	testlib "knative.dev/eventing/test/lib"
	"knative.dev/eventing/test/lib/setupclientoptions"
	"knative.dev/pkg/test/zipkin"
)

const (
	recordEventsAPIPodName  = "api-server-source-logger-pod"
	recordEventsPingPodName = "ping-source-logger-pod"
)

var channelTestRunner testlib.ComponentsTestRunner
var sourcesTestRunner testlib.ComponentsTestRunner
var brokerTestRunner testlib.ComponentsTestRunner
var brokerClass string

func TestMain(m *testing.M) {
	os.Exit(func() int {
		test.InitializeEventingFlags()
		flag.Parse()
		testlib.ReuseNamespace = test.EventingFlags.ReuseNamespace
		channelTestRunner = testlib.ComponentsTestRunner{
			ComponentFeatureMap: testlib.ChannelFeatureMap,
			ComponentsToTest:    test.EventingFlags.Channels,
		}
		sourcesTestRunner = testlib.ComponentsTestRunner{
			ComponentFeatureMap: testlib.SourceFeatureMap,
			ComponentsToTest:    test.EventingFlags.Sources,
		}
		brokerTestRunner = testlib.ComponentsTestRunner{
			ComponentFeatureMap: testlib.BrokerFeatureMap,
			ComponentsToTest:    test.EventingFlags.Brokers,
			ComponentName:       test.EventingFlags.BrokerName,
			ComponentNamespace:  test.EventingFlags.BrokerNamespace,
		}
		brokerClass = test.BrokerClass

		addSourcesInitializers()
		// Any tests may SetupZipkinTracing, it will only actually be done once. This should be the ONLY
		// place that cleans it up. If an individual test calls this instead, then it will break other
		// tests that need the tracing in place.
		defer zipkin.CleanupZipkinTracingSetup(log.Printf)

		exit := m.Run()
		// Collect logs only when test failed.
		if exit != 0 {
			testlib.ExportLogs(testlib.SystemLogsDir, system.Namespace())
		}
		return exit
	}())
}

func addSourcesInitializers() {

	ctx := context.Background()

	apiSrcName := strings.ToLower(testlib.ApiServerSourceTypeMeta.Kind)
	pingSrcName := strings.ToLower(testlib.PingSourceTypeMeta.Kind)
	sourcesTestRunner.AddComponentSetupClientOption(
		testlib.ApiServerSourceTypeMeta,
		setupclientoptions.ApiServerSourceV1ClientSetupOption(
			ctx, apiSrcName, "Reference",
			recordEventsAPIPodName),
	)
	sourcesTestRunner.AddComponentSetupClientOption(
		testlib.PingSourceTypeMeta,
		setupclientoptions.PingSourceV1B2ClientSetupOption(
			ctx, pingSrcName, recordEventsPingPodName),
	)
}
